'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Grid Component for uxcore
 * @author zhouquan.yezq
 *
 * Copyright 2014-2015, UXCore Team, Alinw.
 * All rights reserved.
 */

var SearchBar = require('./SearchBar');
var classnames = require('classnames');
var Button = require('uxcore-button');
var React = require('react');

var ActionBar = function (_React$Component) {
  _inherits(ActionBar, _React$Component);

  function ActionBar() {
    _classCallCheck(this, ActionBar);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  /**
  *  convert ActionBar config from hash to array
  */

  ActionBar.prototype.getActionItem = function getActionItem(config) {
    var items = [];
    if (config instanceof Array) {
      items = config;
    } else if ((typeof config === 'undefined' ? 'undefined' : _typeof(config)) === 'object') {
      Object.keys(config).forEach(function (item) {
        if (config.hasOwnProperty(item)) {
          items.push({
            title: item,
            callback: config[item]
          });
        }
      });
    }
    return items;
  };

  ActionBar.prototype.handleSearch = function handleSearch(value) {
    this.props.onSearch(value);
  };

  ActionBar.prototype.renderActionBtn = function renderActionBtn(item, index) {
    var me = this;
    var itemProps = {
      className: me.props.prefixCls + '-item',
      onClick: item.callback || function () {},
      key: index
    };
    if (!!item.render && typeof item.render === 'function') {
      return React.createElement(
        'div',
        itemProps,
        item.render(item.title)
      );
    }
    return React.createElement(
      Button,
      _extends({ type: 'outline', size: 'medium' }, itemProps),
      item.title
    );
  };

  ActionBar.prototype.renderSearchBar = function renderSearchBar() {
    if (this.props.showSearch) {
      var me = this;
      var searchBarProps = {
        onSearch: me.handleSearch.bind(me),
        key: 'searchbar',
        placeholder: me.props.searchBarPlaceholder
      };
      return React.createElement(SearchBar, searchBarProps);
    }
    return null;
  };

  ActionBar.prototype.render = function render() {
    var _classnames;

    var me = this;
    var props = this.props;
    var barConfig = props.actionBarConfig;
    return React.createElement(
      'div',
      {
        className: classnames((_classnames = {}, _defineProperty(_classnames, props.prefixCls, props.prefixCls), _defineProperty(_classnames, 'fn-clear', true), _classnames))
      },
      me.getActionItem(barConfig).map(function (item, index) {
        return me.renderActionBtn(item, index);
      }),
      me.renderSearchBar()
    );
  };

  return ActionBar;
}(React.Component);

ActionBar.propTypes = {
  showSearch: React.PropTypes.bool,
  onSearch: React.PropTypes.func
};

ActionBar.defaultProps = {
  prefixCls: 'kuma-list-actionbar'
};

module.exports = ActionBar;