'use strict';

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * List Component for uxcore
 * @author eternalsky
 *
 * Copyright 2015-2016, Uxcore Team, Alinw.
 * All rights reserved.
 */
var React = require('react');
var ActionBar = require('./ActionBar');
var ListCore = require('./ListCore');
var Pagination = require('uxcore-pagination');
var deepcopy = require('deepcopy');
var deepEqual = require('deep-equal');
var NattyFetch = require('natty-fetch');
var assign = require('object-assign');
var classnames = require('classnames');

var List = function (_React$Component) {
  _inherits(List, _React$Component);

  function List(props) {
    _classCallCheck(this, List);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.data = deepcopy(props.data);
    _this.handleActionBarSearch = _this.handleActionBarSearch.bind(_this);
    _this.handlePagerChange = _this.handlePagerChange.bind(_this);
    _this.handlePagerSizeChange = _this.handlePagerSizeChange.bind(_this);
    _this.processData = _this.processData.bind(_this);
    _this.state = {
      data: _this.data,
      pageSize: props.pageSize
    };
    return _this;
  }

  List.prototype.componentWillMount = function componentWillMount() {
    this.fetchData();
  };

  List.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var newState = {};
    if (!deepEqual(nextProps.data, this.data)) {
      this.data = deepcopy(nextProps.data);
      newState.data = this.data;
    }
    if (nextProps.pageSize !== this.props.pageSize) {
      newState.pageSize = nextProps.pageSize;
    }
    this.setState(newState);
  };

  List.prototype.componentWillUnmount = function componentWillUnmount() {
    var me = this;
    if (me.fetch) {
      me.fetch.abort();
    }
  };

  /**
   * only call this method in fetchData()
   */


  List.prototype.processData = function processData(data) {
    var me = this;
    var processData = me.props.processData;

    return processData(data);
  };

  List.prototype.handleActionBarSearch = function handleActionBarSearch(key) {
    var me = this;
    me.setState({
      searchTxt: key
    }, function () {
      me.fetchData('search');
    });
  };

  List.prototype.handlePagerSizeChange = function handlePagerSizeChange(currentPage, pageSize) {
    var me = this;
    me.setState({
      pageSize: pageSize
    }, function () {
      me.fetchData('pagination');
    });
  };

  List.prototype.handlePagerChange = function handlePagerChange(currentPage) {
    var me = this;
    var data = assign({}, me.state.data);
    data.currentPage = currentPage;
    me.setState({
      data: data
    }, function () {
      me.fetchData('pagination');
    });
  };

  List.prototype.fetchData = function fetchData(from) {
    var me = this;
    var _me$props = me.props;
    var fetchUrl = _me$props.fetchUrl;
    var fitResponse = _me$props.fitResponse;
    var beforeFetch = _me$props.beforeFetch;
    var onFetchError = _me$props.onFetchError;
    var fetchParams = _me$props.fetchParams;
    var _me$state = me.state;
    var data = _me$state.data;
    var pageSize = _me$state.pageSize;
    var searchTxt = _me$state.searchTxt;

    var isJsonp = me.props.isJsonp === undefined ? /\.jsonp/.test(fetchUrl) : me.props.isJsonp;
    var fetchData = assign({
      currentPage: data.currentPage,
      pageSize: pageSize,
      searchTxt: searchTxt
    }, fetchParams);
    if (fetchUrl) {
      me.fetch = NattyFetch.create({
        url: fetchUrl,
        data: beforeFetch(fetchData, from),
        fit: function fit(response) {
          return fitResponse(response);
        },
        processData: me.processData,
        jsonp: isJsonp
      });
      me.fetch().then(function (content) {
        me.setState({
          data: content
        });
      })["catch"](function (err) {
        onFetchError(err);
      });
    }
  };

  List.prototype.renderActionBar = function renderActionBar() {
    var me = this;
    var _me$props2 = me.props;
    var showSearch = _me$props2.showSearch;
    var actionBar = _me$props2.actionBar;
    var searchBarPlaceholder = _me$props2.searchBarPlaceholder;

    var actionBarProps = {
      onSearch: this.handleActionBarSearch,
      key: 'actionbar',
      actionBarConfig: actionBar,
      showSearch: showSearch,
      searchBarPlaceholder: searchBarPlaceholder
    };
    return React.createElement(ActionBar, actionBarProps);
  };

  List.prototype.renderListCore = function renderListCore() {
    var me = this;
    var data = me.state.data || {};
    return React.createElement(ListCore, {
      data: data.data,
      renderItem: me.props.renderItem,
      locale: me.props.locale
    });
  };

  List.prototype.renderPager = function renderPager() {
    var me = this;
    var data = me.state.data;
    var _me$props3 = me.props;
    var prefixCls = _me$props3.prefixCls;
    var showPager = _me$props3.showPager;
    var locale = _me$props3.locale;
    var showPagerTotal = _me$props3.showPagerTotal;
    var pageSize = _me$props3.pageSize;
    var pagerSizeOptions = _me$props3.pagerSizeOptions;
    /**
     *
     * data structure
     * {
     *   data: [],
     *   currentPage: 1,
     *   totalCount: 30
     * }
     *
     */

    if (showPager && data && data.totalCount && data.currentPage) {
      var _React$createElement;

      return React.createElement(
        'div',
        { className: prefixCls + '-pager' },
        React.createElement(Pagination, (_React$createElement = {
          total: data.totalCount,
          locale: locale,
          showSizeChanger: true,
          showQuickJumper: true,
          showTotal: showPagerTotal
        }, _defineProperty(_React$createElement, 'total', data.totalCount), _defineProperty(_React$createElement, 'onShowSizeChange', me.handlePagerSizeChange), _defineProperty(_React$createElement, 'onChange', me.handlePagerChange), _defineProperty(_React$createElement, 'current', data.currentPage), _defineProperty(_React$createElement, 'pageSize', pageSize), _defineProperty(_React$createElement, 'sizeOptions', pagerSizeOptions), _React$createElement))
      );
    }
    return null;
  };

  List.prototype.render = function render() {
    var _classnames;

    var me = this;
    var _me$props4 = me.props;
    var prefixCls = _me$props4.prefixCls;
    var width = _me$props4.width;
    var height = _me$props4.height;
    var className = _me$props4.className;

    return React.createElement(
      'div',
      {
        className: classnames((_classnames = {}, _defineProperty(_classnames, '' + prefixCls, true), _defineProperty(_classnames, className, !!className), _classnames)),
        style: { width: width, height: height }
      },
      me.renderActionBar(),
      me.renderListCore(),
      me.renderPager()
    );
  };

  return List;
}(React.Component);

List.defaultProps = {
  prefixCls: 'kuma-list',
  locale: 'zh-cn',
  fetchParams: {},
  searchBarPlaceholder: '请输入',
  showPagerTotal: true,
  showSearch: false,
  showPager: true,
  pageSize: 10,
  pagerSizeOptions: [10, 20, 30, 40],
  data: {},
  renderItem: function renderItem() {},
  fitResponse: function fitResponse(response) {
    return response;
  },
  beforeFetch: function beforeFetch(data) {
    return data;
  },
  processData: function processData(data) {
    return data;
  },
  onFetchError: function onFetchError(err) {
    console.error(err.stack);
  },
  width: 'auto',
  height: 'auto'
};

// http://facebook.github.io/react/docs/reusable-components.html
List.propTypes = {
  prefixCls: React.PropTypes.string,
  className: React.PropTypes.string,
  locale: React.PropTypes.string,
  emptyText: React.PropTypes.string,
  fetchUrl: React.PropTypes.string,
  fetchParams: React.PropTypes.object,
  searchBarPlaceholder: React.PropTypes.string,
  data: React.PropTypes.object,
  renderItem: React.PropTypes.func,
  fitResponse: React.PropTypes.func,
  beforeFetch: React.PropTypes.func,
  processData: React.PropTypes.func,
  onFetchError: React.PropTypes.func,
  isJsonp: React.PropTypes.bool,
  showSearch: React.PropTypes.bool,
  showPager: React.PropTypes.bool,
  showPagerTotal: React.PropTypes.bool,
  pagerSizeOptions: React.PropTypes.array,
  pageSize: React.PropTypes.number,
  width: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number]),
  height: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number]),
  actionBar: React.PropTypes.oneOfType([React.PropTypes.array, React.PropTypes.object])
};

List.displayName = 'List';

module.exports = List;